@@ -3,17 +3,17 @@
 lazyarray is a Python package that provides a lazily-evaluated numerical array
 class, ``larray``, based on and compatible with NumPy arrays.
 
-Copyright Andrew P. Davison, Joël Chavas and Elodie Legouée (CNRS), 2012-2019
+Copyright Andrew P. Davison, Joël Chavas and Elodie Legouée (CNRS), 2012-2020
 """
 
-from __future__ import division
-import numpy
+import numbers
 import operator
 from copy import deepcopy
 import collections
-from functools import wraps
+from functools import wraps, reduce
 import logging
 
+import numpy as np
 try:
     from scipy import sparse
     from scipy.sparse import bsr_matrix, coo_matrix, csc_matrix, csr_matrix, dia_matrix, dok_matrix, lil_matrix
@@ -22,24 +22,7 @@ except ImportError:
     have_scipy = False
 
 
-__version__ = "0.3.3"
-
-# stuff for Python 3 compatibility
-try:
-    long
-except NameError:
-    long = int
-
-try:
-    reduce
-except NameError:
-    from functools import reduce
-
-try:
-    basestring
-except NameError:
-    basestring = str
-
+__version__ = "0.5.0"
 
 logger = logging.getLogger("lazyarray")
 
@@ -51,7 +34,7 @@ def check_shape(meth):
     """
     @wraps(meth)
     def wrapped_meth(self, val):
-        if isinstance(val, (larray, numpy.ndarray)) and val.shape:
+        if isinstance(val, (larray, np.ndarray)) and val.shape:
             if val.shape != self._shape:
                 raise ValueError("shape mismatch: objects cannot be broadcast to a single shape")
         return meth(self, val)
@@ -68,7 +51,7 @@ def requires_shape(meth):
 
 
 def full_address(addr, full_shape):
-    if not (isinstance(addr, numpy.ndarray) and addr.dtype == bool and addr.ndim == len(full_shape)):
+    if not (isinstance(addr, np.ndarray) and addr.dtype == bool and addr.ndim == len(full_shape)):
         if not isinstance(addr, tuple):
             addr = (addr,)
         if len(addr) < len(full_shape):
@@ -84,7 +67,7 @@ def partial_shape(addr, full_shape):
     Calculate the size of the sub-array represented by `addr`
     """
     def size(x, max):
-        if isinstance(x, (int, long, numpy.integer)):
+        if isinstance(x, (int, np.integer)):
             return None
         elif isinstance(x, slice):
             y = min(max, x.stop or max)  # slice limits can go past the bounds
@@ -98,7 +81,7 @@ def partial_shape(addr, full_shape):
             raise TypeError("Unsupported index type %s" % type(x))
 
     addr = full_address(addr, full_shape)
-    if isinstance(addr, numpy.ndarray) and addr.dtype == bool:
+    if isinstance(addr, np.ndarray) and addr.dtype == bool:
         return (addr.sum(),)
     elif all(isinstance(x, collections.Sized) for x in addr):
         return (len(addr[0]),)
@@ -145,6 +128,25 @@ def lazy_unary_operation(name):
     return op
 
 
+def is_array_like(value):
+    # False for numbers, generators, functions, iterators
+    if not isinstance(value, collections.Sized):
+        return False
+    if sparse.issparse(value):
+        return True
+    if isinstance(value, collections.Mapping):
+        # because we may wish to have lazy arrays in which each
+        # item is a dict, for example
+        return False
+    if getattr(value, "is_lazyarray_scalar", False):
+        # for user-defined classes that are "Sized" but that should
+        # be treated as individual elements in a lazy array
+        # the attribute "is_lazyarray_scalar" can be defined with value
+        # True.
+        return False
+    return True
+
+
 
 class larray(object):
     """
@@ -166,7 +168,7 @@ class larray(object):
         """
         Create a new lazy array.
 
-        `value` : may be an int, long, float, bool, NumPy array, iterator,
+        `value` : may be an int, float, bool, NumPy array, iterator,
                   generator or a function, `f(i)` or `f(i,j)`, depending on the
                   dimensions of the array.
 
@@ -177,7 +179,7 @@ class larray(object):
 
         self.dtype = dtype
         self.operations = []
-        if isinstance(value, basestring):
+        if isinstance(value, str):
             raise TypeError("An larray cannot be created from a string")
         elif isinstance(value, larray):
             if shape is not None and value.shape is not None:
@@ -187,20 +189,23 @@ class larray(object):
             self.dtype = dtype or value.dtype
             self.operations = value.operations  # should deepcopy?
 
-        elif isinstance(value, collections.Sized):  # False for numbers, generators, functions, iterators
+        elif is_array_like(value):  # False for numbers, generators, functions, iterators
             if have_scipy and sparse.issparse(value):  # For sparse matrices
                 self.dtype = dtype or value.dtype
-            elif not isinstance(value, numpy.ndarray):
-                value = numpy.array(value, dtype=dtype)
+            elif not isinstance(value, np.ndarray):
+                value = np.array(value, dtype=dtype)
             elif dtype is not None:
-               assert numpy.can_cast(value.dtype, dtype, casting='safe')  # or could convert value to the provided dtype
-            if shape and value.shape != shape:
+               assert np.can_cast(value.dtype, dtype, casting='safe')  # or could convert value to the provided dtype
+            if shape and value.shape and value.shape != shape:
                 raise ValueError("Array has shape %s, value has shape %s" % (shape, value.shape))
-            self._shape = value.shape
+            if value.shape:
+                self._shape = value.shape
+            else:
+                self._shape = shape
             self.base_value = value
 
         else:
-            assert numpy.isreal(value)  # also True for callables, generators, iterators
+            assert np.isreal(value)  # also True for callables, generators, iterators
             self._shape = shape
             if dtype is None or isinstance(value, dtype):
                 self.base_value = value
@@ -211,7 +216,19 @@ class larray(object):
                     self.base_value = value
 
     def __eq__(self, other):
-        return self.base_value == other.base_value and self.operations == other.operations and self._shape == other.shape
+        if isinstance(other, self.__class__):
+            return self.base_value == other.base_value and self.operations == other.operations and self._shape == other.shape
+        elif isinstance(other, numbers.Number):
+            if len(self.operations) == 0:
+                if isinstance(self.base_value, numbers.Number):
+                    return self.base_value == other
+                elif isinstance(self.base_value, np.ndarray):
+                    return (self.base_value == other).all()
+            # todo: we could perform the evaluation ourselves, but that could have a performance hit
+            raise Exception("You will need to evaluate this lazyarray before checking for equality")
+        else:
+            # todo: add support for NumPy arrays
+            raise TypeError("Cannot at present compare equality of lazyarray and {}".format(type(other)))
 
     def __deepcopy__(self, memo):
         obj = type(self).__new__(type(self))
@@ -226,7 +243,7 @@ class larray(object):
         obj.dtype = self.dtype
         obj.operations = []
         for f, arg in self.operations:
-            if isinstance(f, numpy.ufunc):
+            if isinstance(f, np.ufunc):
                 obj.operations.append((f, deepcopy(arg)))
             else:
                 obj.operations.append((deepcopy(f), deepcopy(arg)))
@@ -240,7 +257,7 @@ class larray(object):
 
     def _set_shape(self, value):
         if (hasattr(self.base_value, "shape") and
-                self.base_value.shape and   # values of type numpy.float have an empty shape
+                self.base_value.shape and   # values of type np.float have an empty shape
                     self.base_value.shape != value):
             raise ValueError("Lazy array has fixed shape %s, cannot be changed to %s" % (self.base_value.shape, value))
         self._shape = value
@@ -274,7 +291,7 @@ class larray(object):
     @property
     def is_homogeneous(self):
         """True if all the elements of the array are the same."""
-        hom_base = isinstance(self.base_value, (int, long, numpy.integer, float, bool)) \
+        hom_base = isinstance(self.base_value, (int, np.integer, float, bool)) \
                    or type(self.base_value) == self.dtype \
                    or (isinstance(self.dtype, type) and isinstance(self.base_value, self.dtype))
         hom_ops = all(obj.is_homogeneous for f, obj in self.operations if isinstance(obj, larray))
@@ -289,7 +306,7 @@ class larray(object):
     def _homogeneous_array(self, addr):
         self.check_bounds(addr)
         shape = self._partial_shape(addr)
-        return numpy.ones(shape, type(self.base_value))
+        return np.ones(shape, type(self.base_value))
 
     def _full_address(self, addr):
         return full_address(addr, self._shape)
@@ -298,28 +315,28 @@ class larray(object):
         self.check_bounds(addr)
 
         def axis_indices(x, max):
-            if isinstance(x, (int, long, numpy.integer)):
+            if isinstance(x, (int, np.integer)):
                 return x
             elif isinstance(x, slice):  # need to handle negative values in slice
-                return numpy.arange((x.start or 0),
+                return np.arange((x.start or 0),
                                     (x.stop or max),
                                     (x.step or 1),
                                     dtype=int)
             elif isinstance(x, collections.Sized):
                 if hasattr(x, 'dtype') and x.dtype == bool:
-                    return numpy.arange(max)[x]
+                    return np.arange(max)[x]
                 else:
-                    return numpy.array(x)
+                    return np.array(x)
             else:
                 raise TypeError("Unsupported index type %s" % type(x))
         addr = self._full_address(addr)
-        if isinstance(addr, numpy.ndarray) and addr.dtype == bool:
+        if isinstance(addr, np.ndarray) and addr.dtype == bool:
             if addr.ndim == 1:
-                return (numpy.arange(self._shape[0])[addr],)
+                return (np.arange(self._shape[0])[addr],)
             else:
                 raise NotImplementedError()
         elif all(isinstance(x, collections.Sized) for x in addr):
-            indices = [numpy.array(x) for x in addr]
+            indices = [np.array(x) for x in addr]
             return indices
         else:
             indices = [axis_indices(x, max) for (x, max) in zip(addr, self._shape)]
@@ -328,7 +345,7 @@ class larray(object):
             elif len(indices) == 2:
                 if isinstance(indices[0], collections.Sized):
                     if isinstance(indices[1], collections.Sized):
-                        mesh_xy = numpy.meshgrid(*indices)
+                        mesh_xy = np.meshgrid(*indices)
                         return (mesh_xy[0].T, mesh_xy[1].T)  # meshgrid works on (x,y), not (i,j)
                 return indices
             else:
@@ -353,16 +370,16 @@ class larray(object):
                 base_val = self.base_value
             else:
                 base_val = self._homogeneous_array(addr) * self.base_value
-        elif isinstance(self.base_value, (int, long, numpy.integer, float, bool)):
+        elif isinstance(self.base_value, (int, np.integer, float, bool)):
             base_val = self._homogeneous_array(addr) * self.base_value
-        elif isinstance(self.base_value, numpy.ndarray):
+        elif isinstance(self.base_value, np.ndarray):
             base_val = self.base_value[addr]
         elif have_scipy and sparse.issparse(self.base_value):  # For sparse matrices larr[2, :]
             base_val = self.base_value[addr]
         elif callable(self.base_value):
             indices = self._array_indices(addr)
             base_val = self.base_value(*indices)
-            if isinstance(base_val, numpy.ndarray) and base_val.shape == (1,):
+            if isinstance(base_val, np.ndarray) and base_val.shape == (1,):
                 base_val = base_val[0]
         elif hasattr(self.base_value, "lazily_evaluate"):
             base_val = self.base_value.lazily_evaluate(addr, shape=self._shape)
@@ -392,7 +409,7 @@ class larray(object):
             return hasattr(arr, 'dtype') and arr.dtype == bool
 
         def check_axis(x, size):
-            if isinstance(x, (int, long, numpy.integer)):
+            if isinstance(x, (int, np.integer)):
                 lower = upper = x
             elif isinstance(x, slice):
                 lower = x.start or 0
@@ -417,7 +434,7 @@ class larray(object):
             if (lower < -size) or (upper >= size):
                 raise IndexError("Index out of bounds")
         full_addr = self._full_address(addr)
-        if isinstance(addr, numpy.ndarray) and addr.dtype == bool:
+        if isinstance(addr, np.ndarray) and addr.dtype == bool:
             if len(addr.shape) > len(self._shape):
                 raise IndexError("Too many indices for array")
             for xmax, size in zip(addr.shape, self._shape):
@@ -434,7 +451,7 @@ class larray(object):
         where `x` will be a scalar or a numpy array.
 
         >>> m = larray(4, shape=(2,2))
-        >>> m.apply(numpy.sqrt)
+        >>> m.apply(np.sqrt)
         >>> m.evaluate()
         array([[ 2.,  2.],
                [ 2.,  2.]])
@@ -468,13 +485,13 @@ class larray(object):
             if simplify:
                 x = self.base_value
             else:
-                x = self.base_value * numpy.ones(self._shape, dtype=self.dtype)
-        elif isinstance(self.base_value, (int, long, numpy.integer, float, bool, numpy.bool_)):
-            x = self.base_value * numpy.ones(self._shape, dtype=self.dtype)
-        elif isinstance(self.base_value, numpy.ndarray):
+                x = self.base_value * np.ones(self._shape, dtype=self.dtype)
+        elif isinstance(self.base_value, (int, np.integer, float, bool, np.bool_)):
+            x = self.base_value * np.ones(self._shape, dtype=self.dtype)
+        elif isinstance(self.base_value, np.ndarray):
             x = self.base_value
         elif callable(self.base_value):
-            x = numpy.array(numpy.fromfunction(self.base_value, shape=self._shape, dtype=int), dtype=self.dtype)
+            x = np.array(np.fromfunction(self.base_value, shape=self._shape, dtype=int), dtype=self.dtype)
         elif hasattr(self.base_value, "lazily_evaluate"):
             x = self.base_value.lazily_evaluate(shape=self._shape)
         elif isinstance(self.base_value, VectorizedIterable):
@@ -484,11 +501,11 @@ class larray(object):
         elif have_scipy and sparse.issparse(self.base_value):  # For sparse matrices
             if empty_val!=0:
                 x = self.base_value.toarray((sparse.csc_matrix))
-                x = numpy.where(x, x, numpy.nan)
+                x = np.where(x, x, np.nan)
             else:
                 x = self.base_value.toarray((sparse.csc_matrix))
         elif isinstance(self.base_value, collections.Iterator):
-            x = numpy.fromiter(self.base_value, dtype=self.dtype or float, count=self.size)
+            x = np.fromiter(self.base_value, dtype=self.dtype or float, count=self.size)
             if x.shape != self._shape:
                 x = x.reshape(self._shape)
         else:
@@ -537,6 +554,15 @@ class larray(object):
     __abs__ = lazy_unary_operation('abs')
 
 
+class VectorizedIterable(object):
+    """
+    Base class for any class which has a method `next(n)`, i.e., where you
+    can choose how many values to return rather than just returning one at a
+    time.
+    """
+    pass
+
+
 def _build_ufunc(func):
     """Return a ufunc that works with lazy arrays"""
     def larray_compatible_ufunc(x):
@@ -549,19 +575,28 @@ def _build_ufunc(func):
     return larray_compatible_ufunc
 
 
-class VectorizedIterable(object):
-    """
-    Base class for any class which has a method `next(n)`, i.e., where you
-    can choose how many values to return rather than just returning one at a
-    time.
-    """
-    pass
+def _build_ufunc_2nd_arg(func):
+    """Return a ufunc taking a second, non-array argument, that works with lazy arrays"""
+    def larray_compatible_ufunc2(x1, x2):
+        if not isinstance(x2, numbers.Number):
+            raise TypeError("lazyarry ufuncs do not accept an array as the second argument")
+        if isinstance(x1, larray):
+            def partial(x):
+                return func(x, x2)
+            y = deepcopy(x1)
+            y.apply(partial)
+            return y
+        else:
+            return func(x1, x2)
+    return larray_compatible_ufunc2
 
 
 # build lazy-array compatible versions of NumPy ufuncs
 namespace = globals()
-for name in dir(numpy):
-    obj = getattr(numpy, name)
-    if isinstance(obj, numpy.ufunc):
-        namespace[name] = _build_ufunc(obj)
-
+for name in dir(np):
+    obj = getattr(np, name)
+    if isinstance(obj, np.ufunc) and name not in namespace:
+        if name in ("power", "fmod", "arctan2, hypot, ldexp, maximum, minimum"):
+            namespace[name] = _build_ufunc_2nd_arg(obj)
+        else:
+            namespace[name] = _build_ufunc(obj)
